#include "XC2dWaveprop.c"

// dimensional information ( t,x,y,z,s, note that x,y,z info are after tapering/padding)
int   XC2dRTM_nt,XC2dRTM_nx,XC2dRTM_nz,XC2dRTM_ns;
float XC2dRTM_ot,XC2dRTM_ox,XC2dRTM_oz;
float XC2dRTM_dt,XC2dRTM_dx,XC2dRTM_dz;
float *XC2dRTM_sdep, *XC2dRTM_rdep;
int *XC2dRTM_isdep, *XC2dRTM_irdep;
//tapering/padding information ( in x,y,z directions) 
int   XC2dRTM_nxtape,XC2dRTM_nztape;
int   XC2dRTM_nxt,XC2dRTM_nzt,XC2dRTM_jtmv;
float *XC2dRTM_vel;
float *XC2dRTM_dxinvveldt2;
float XC2dRTM_vmax;
//incremental z range of wavefield to be calculated as t increases
int   XC2dRTM_izmax_inc;

#define zxloc(iz,ix) ((iz)+(ix)*XC2dRTM_nz)
#define Errmsg(str)  {fprintf(stderr,str);return 1;}
#define Errmsg2(str,str1)  {fprintf(stderr,str,str1);return 1;}
#define Errmsg3(str,str1,str2)  {fprintf(stderr,str,str1,str2);return 1;}

int XC2dRTM_Init( int ntin,  int nxin, int nzin, int nsin,  \
                      float otin,float oxin,float ozin,          \
                      float dtin,float dxin,float dzin,          \
                      int nxtapein, int nztapein,             \
                      float* sdepin, float *rdepin, float* velin,           \
                      int needwvmvin, int jtmvin );
int XC2dRTM_op(int tsign, int is, float *src, float *dat, double *imag);
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
int XC2dRTM_Init( int ntin,  int nxin, int nzin, int nsin,  \
                      float otin,float oxin,float ozin,          \
                      float dtin,float dxin,float dzin,          \ 
                      int nxtapein, int nztapein,             \ 
                      float* sdepin, float *rdepin, float* velin,           \
                      int needwvmvin, int jtmvin )
{
    int ix,iy,iz,stat;
    float temp,v,vmax;
    // take input parameters
    XC2dRTM_nt=ntin;
    XC2dRTM_nx=nxin;
    XC2dRTM_nz=nzin;
    XC2dRTM_ns=nsin;

    XC2dRTM_ot=otin;
    XC2dRTM_ox=oxin;
    XC2dRTM_oz=ozin;

    XC2dRTM_dt=dtin;
    XC2dRTM_dx=dxin;
    XC2dRTM_dz=dzin;

    XC2dRTM_nxtape=nxtapein;
    XC2dRTM_nztape=nztapein;
    XC2dRTM_jtmv=jtmvin;

    XC2dRTM_sdep=sdepin;
    XC2dRTM_rdep=rdepin;
    XC2dRTM_vel=velin;

    // determine if padded region is reasonable
    if (nxin<=2*nxtapein)
       WpErrmsg("Error, nx less or equal than twice of padded x reigon\n")
    if (nzin<=2*nztapein)
       WpErrmsg("Error, nz less or equal than twice of padded z reigon\n")

    // determine if dx==dz
    if (  XC2dRTM_dz!=XC2dRTM_dx )
       Errmsg("ERROR, dz not equal to dx, exit!!!\n")
    else 
    {
       // produce (vel*dt/dx)**2 and get vmax/izmax_inc on the way
       vmax=0.;
       XC2dRTM_isdep=(int*)malloc(nxin*sizeof(int));
       XC2dRTM_irdep=(int*)malloc(nxin*sizeof(int));
       XC2dRTM_dxinvveldt2=(float*)malloc(nxin*nzin*sizeof(float));
       for (ix=0;ix<nxin;ix++)
          for (iz=0;iz<nzin;iz++)
          {
             v=XC2dRTM_vel[ zxloc(iz,ix) ];
             temp= v * XC2dRTM_dt/XC2dRTM_dx;
             XC2dRTM_dxinvveldt2[ zxloc(iz,ix) ]=1./(temp*temp);
             if ( v  > vmax)
                vmax = v;
             if (temp>0.86)
                Errmsg2("ERROR, stability condition for wavepropagation is violated, temp=%f which is larger than 0.86\n",temp)
          }
       XC2dRTM_izmax_inc= vmax*XC2dRTM_dt/XC2dRTM_dz + 1;
    }
    // initilize wave propagation module
    int freesurf=0;
    long memcnt=0; 
    stat=XC2dWaveprop_Init(ntin,nxin,nzin,nxtapein,nztapein,needwvmvin,jtmvin, freesurf, &memcnt);
    if (stat==1)
       Errmsg("Error in XC2dWaveprop_Init, program terminated!\n")

    return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////
int XC2dRTM_op(int tsign, int is, float *src, float *dat, double *imag)
{
  float *sdep,*rdep;
  int ix;
  int nx;
  int izmax_inc,stat;
  float dz;

  //VerbMsg("Calculating sdep\n")
  sdep=&XC2dRTM_sdep[is*XC2dRTM_nx];
  rdep=&XC2dRTM_rdep[is*XC2dRTM_nx];
 
  nx=XC2dRTM_nx;
  dz=XC2dRTM_dz;
  // convert float depth (sdep, rdep) into integer index depth(isdep, irdep) 
 
  //VerbMsg("Calculating isdep\n")
  for (ix=0;ix<nx;ix++)
     XC2dRTM_isdep[ ix ]=(sdep[ ix ]-XC2dRTM_oz)/dz+XC2dRTM_nztape;

  //VerbMsg("Calculating irdep\n")
  for (ix=0;ix<nx;ix++)
     XC2dRTM_irdep[ ix ]=(rdep[ ix ]-XC2dRTM_oz)/dz+XC2dRTM_nztape;

   fprintf(stderr,"isdep=%d , irdep=%d\n",XC2dRTM_isdep[0],XC2dRTM_irdep[0]);
  // wave propgation
  izmax_inc=XC2dRTM_izmax_inc;
  stat=XC2dWaveprop_op2_rndbnd_imag(XC2dRTM_isdep,XC2dRTM_irdep, src, XC2dRTM_dxinvveldt2, izmax_inc, dat, imag, (XC2dRTM_dt*XC2dRTM_jtmv) );
//stat=XC2dWaveprop_op( tsign, XC2dRTM_isdep, XC2dRTM_irdep, src, XC2dRTM_dxinvveldt2, izmax_inc, dat, wvmv );

  if (stat==1) 
     Errmsg("Error in XC2dWaveprop_op, program terminated!\n")   
  
  return 0;
}
