#include "stdlib.h"
#include "math.h"
#include "string.h"
#include <sep_pars_external.h>
#include <seplib.h>
#include <sep3dc.h>
#include "fftw3.h"
#include "XC2dRTM.c"

int main(int argc, char** argv)
{
    int ndim,i,n;
    float o,d;
    char temp_char[256];
    int   nt, nx, nz, ns;
    float ot, ox, oz;
    float dt, dx, dz;
    int   it, ix, iz, is;
    int   nxtape,nztape;
    int   needwvmv,jtmv;
    int   stat,tsign;
    float *vel, *src, *data, *sdep, *rdep, *wvmv, *imag;
    double *imagd;
    sep3d outfile; 
    time_t time1,time2;

    initpar(argc,argv);
    // read t,x,y from source file
   
    if (0==getch("nxtape","i", &nxtape)) nxtape=50;
    if (0==getch("nztape","i", &nztape)) nztape=50;
    if (0==getch("needwvmv","i", &needwvmv)) needwvmv=0;
    if (0==getch("jtmv","i", &jtmv)) jtmv=10;
    if (0==getch("tsign","i", &tsign)) tsign=1;
    if(0!=sep_get_number_data_axes("in",&ndim))
       seperr("unable to get dimension from src!\n");
    
    for(i=1;i<=ndim;i++)
    {
       if(0!=sep_get_data_axis_par("in",&i,&n,&o,&d,temp_char))
       seperr("unable to get parameters from src!\n");
       if(i==1)
       {
          nt=n;
          dt=d;
          ot=o;
       }
       else if(i==2)
       {
          nx=n+2*nxtape;
          dx=d;
          ox=o-nxtape*d;
       }
       if (ndim==2) 
          ns=1; 
       else if(i==3)
          ns=n;
    }

    // read z info from vel
    if(0!=sep_get_number_data_axes("vel",&ndim))
       seperr("unable to get dimension from vel file!\n");
    for(i=1;i<=ndim;i++)
    {
       if(0!=sep_get_data_axis_par("vel",&i,&n,&o,&d,temp_char))
       seperr("unable to get parameters from vel file!\n");
       if(i==1)
       {
          nz=n;
          dz=d;
          oz=o;
       }
       else if(i==2)
       {
          if (nx!=n)
          Errmsg3("nx in vel %d different from nx %d in src, program terminated!\n",nx,n)
          if (dx!=d)
          Errmsg3("dx in vel %f different from dx %f in src, program terminated!\n",dx,d)
          if (ox!=o)
          Errmsg3("ox in vel %f different from ox %f in src, program terminated!\n",ox,o)
       }
    }

    // readin other parameters

    if (nx<=2*nxtape)
       Errmsg3("Error, nx %d less or equal than twice of padded x reigon %d \n",nx,nxtape);
    if (nz<=2*nztape)
       Errmsg3("Error, nz %d less or equal than twice of padded z reigon %d \n",nz,nztape);

    // allocate memories

    vel=(float*)malloc(nx*nz*sizeof(float));
    src=(float*)malloc(nx*nt*ns*sizeof(float));
    data=(float*)malloc(nx*nt*ns*sizeof(float));
    imag=(float*)malloc(nx*nz*ns*sizeof(float));
    imagd=(double*)malloc(nx*nz*sizeof(double));
    sdep=(float*)malloc(nx*ns*sizeof(float));
    rdep=(float*)malloc(nx*ns*sizeof(float));

    for (i=0;i<nx*nt*ns;i++)
       data[i]=0; 

    fprintf(stderr,"nx=%d,nt=%d,ns=%d\n",nx,nt,ns);
    /*fprintf(stderr,"start reading src & data & vel!\n");
    // read in various data files into arrays
    if (((nx-2*nxtape)*nt*ns*4)!=sreed("in",src,(nx-2*nxtape)*nt*ns*4))
       Errmsg2("Error read in src file of size %d \n",nx*nt*ns*4);

    if (((nx-2*nxtape)*nt*ns*4)!=sreed("data",data,(nx-2*nxtape)*nt*ns*4))
       Errmsg2("Error read in data file of size %d \n",nx*nt*ns*4);
    */
    if ((nx*nz*4)!=sreed("vel",vel,nx*nz*4))
       Errmsg2("Error read in vel file of size %d \n", nx*nz*4);

    fprintf(stderr,"done reading src & data & vel!\n");
    for (i=0;i<nx*nz*ns;i++)
       imag[i]=0;
    
    FILE *fp;
       if ((nx*ns*4)!=sreed("sdep",sdep,nx*ns*4))
       {
          fprintf(stderr,"sdep file not found!\n");
          for (i=0;i<nx*ns;i++)
             sdep[i]=2*dz;
       }

       if ((nx*ns*4)!=sreed("rdep",rdep,nx*ns*4))
       {
          fprintf(stderr,"rdep file not found!\n");
          for (i=0;i<nx*ns;i++)
             rdep[i]=2*dz;
       }


    if (needwvmv>0) 
       wvmv=(float*)malloc(nx*nz*((nt-1)/jtmv+1)*sizeof(float));
     
    // Initialize wave propagation 
    VerbMsg("start initialize XC2dRTM_Init\n")
    stat=XC2dRTM_Init( nt,nx,nz,ns,       \
                           ot,ox,oz,          \
                           dt,dx,dz,          \
                           nxtape,nztape, \
                           sdep, rdep, vel,      \
                           needwvmv,jtmv );

    if (stat==1)
       seperr("Error in XC2dRTM_Init,program terminated!\n");
    
    // run wave propagation 
    VerbMsg("start modeling in XC2dRTM_op \n")
    time1=time(NULL);
    for (is=0;is<ns;is++) 
    {
       fprintf(stderr,"start imaging shot %d!\n",is+1);
       if (((nx-2*nxtape)*nt*4)!=sreed("in",src,(nx-2*nxtape)*nt*4))
          Errmsg2("Error read in src file of size %d \n",nx*nt*ns*4);

       if (((nx-2*nxtape)*nt*4)!=sreed("data",data,(nx-2*nxtape)*nt*4))
          Errmsg2("Error read in data file of size %d \n",nx*nt*ns*4);

       for (ix=0;ix<nx*nz;ix++)
          imagd[ix]=0;
       //stat=XC2dRTM_op(tsign,is,&src[is*(nx-2*nxtape)*nt],&data[is*(nx-2*nxtape)*nt],imagd);
       stat=XC2dRTM_op(tsign,is,src,data,imagd);
       for (ix=0;ix<nx*nz;ix++)
          imag[ix+is*nx*nz]=imagd[ix];
    }
    time2=time(NULL);

    fprintf(stderr,"used time for 1 shot is %ld seconds\n", time2-time1);
    if (stat==1)
       seperr("Error in XC2dRTM_op,program terminated!\n");

    //output recorded data
    VerbMsg("start output recorded data \n")
    ndim=3;
    if(0!=init_sep3d_par(&outfile,"OUTPUT","FLOAT","REGULAR",ndim,0))
       seperr("unable to write outfile!\n");
    outfile.o[0]=oz;
    outfile.d[0]=dz;
    outfile.n[0]=nz;
    outfile.o[1]=ox;
    outfile.d[1]=dx;
    outfile.n[1]=nx;
    outfile.o[2]=0;
    outfile.d[2]=1;
    outfile.n[2]=ns;
    sep3d_set_sep3d(&outfile);
    if(0!=sep3dc_write_description("imag",&outfile))
       seperr("unable to output parameters for data\n");

    if( (nz*nx*ns*4)!=srite("imag",imag,4*nz*nx*ns) )
       seperr("trouble writing out data\n");

    //output wave movie if necessary
    if (needwvmv>0)
    {
       ndim=3;
       VerbMsg("start output wave movie \n")
       if(0!=init_sep3d_par(&outfile,"OUTPUT","FLOAT","REGULAR",ndim,0))
          seperr("unable to write outfile!\n");
       outfile.o[0]=oz;
       outfile.d[0]=dz;
       outfile.n[0]=nz;
       outfile.o[1]=ox;
       outfile.d[1]=dx;
       outfile.n[1]=nx;
       outfile.o[2]=ot;
       outfile.d[2]=dt*jtmv;
       outfile.n[2]=nt/jtmv;
       sep3d_set_sep3d(&outfile);
       VerbMsg("start output wave movie header \n" )
       if(0!=sep3dc_write_description("wvmv",&outfile))
          seperr("unable to output parameters for wvmv\n");

       VerbMsg("start output wave movie data body \n" )
       if( (nx*nz*(nt/jtmv)*4)!=srite("wvmv",wvmv,4*nx*nz*(nt/jtmv)) )
          seperr("trouble writing out wvmv\n");
       VerbMsg("done output wave movie data body \n" )
    }
    return 0;
}
