#define min(a,b) (a)<(b)?(a):(b)
#define max(a,b) (a)>(b)?(a):(b)


/**
* Returns the minimum of three values.
* If equal values, choose the value b.
*/
float min3(float a, float b, float c) {
   return b<=a?(b<=c?b:c):(a<=c?a:c);
}


/**
* Non-linear accumulation of alignment errors.
* @param dir accumulation direction, + or -.
* @param b strain is bounded by 1/b.
* @param e input array of alignment errors.
* @param d output array of accumulated errors.
*/
void accumulate(int dir, int b, int nl, int ni, float *e, float *d)
{
   int nlm1 = nl-1;
   int nim1 = ni-1;
   int ib = (dir>0)?0:nim1;
   int ie = (dir>0)?ni:-1;
   int is = (dir>0)?1:-1;
   int i,ji,jb,l,lm1,lp1,kb;
   float dm,di,dp;
   for (i=0;i<nl*ni;i++)
      d[i]=0;
   for (i=ib; i!=ie; i+=is) {
      ji = max(0,min(nim1,i-is));
      jb = max(0,min(nim1,i-is*b));
      for (l=0; l<nl; ++l) {
         lm1 = l-1; if (lm1==-1) lm1 = 0;
         lp1 = l+1; if (lp1==nl) lp1 = nlm1;
         dm = d[lm1+nl*jb];
         di = d[l+nl*ji];
         dp = d[lp1+nl*jb];
         for (kb=ji; kb!=jb; kb-=is) {
            dm += e[lm1+nl*kb];
            dp += e[lp1+nl*kb];
         }
         d[l+nl*i] = min3(dm,di,dp)+e[l+nl*i];
      }
   }
}
/**
* Non-linear accumulation of alignment errors.
* @param dir accumulation direction, + or -.
* @param b strain is bounded by 1/b.
* @param e input array of alignment errors.
* @param d output array of accumulated errors.
*/
void bdsmooth(int nl, int ni, float *e, float *eout, float *ef, float*er)
{
   int dir=1;
   int b=1;
   accumulate(dir,b,nl,ni,e,ef);
   accumulate(-dir,b,nl,ni,e,er);
   int i;
   for (i=0;i<nl*ni;i++)
      eout[i]=ef[i]+er[i]-e[i];
}
/**
* Finds shifts by backtracking within an
* array of accumulated alignment errors.
* Backtracking must be performed in the
* direction opposite that in which the
* alignment errors where accumulated.
* @param dir backtrack direction, + or -.
* @param b strain is bounded by 1/b.
* @param lmin lag for lag index 0.
* @param d input array of accumulated errors.
* @param e input array of alignment errors.
* @param u output array of shifts.
*/
void backtrack(int dir, int b, int lmin, int nl, int ni, float *d, float *e, float *u)
{
   int nlm1 = nl-1;
   int nim1 = ni-1;
   int ib = (dir>0)?0:nim1;
   int ie = (dir>0)?nim1:0;
   int is = (dir>0)?1:-1;
   int ii = ib;
   int il = 0;
   float dl = d[il+nl*ii];
   int jl,ji,jb,ilm1, ilp1, kb;
   float dm,di,dp;
   for (jl=1; jl<nl; ++jl) {
      if (d[jl+nl*ii]<dl) {
         dl = d[jl+nl*ii];
         il = jl;
      }
   }
   u[ii] = il+lmin;
   while (ii!=ie) {
      ji = max(0,min(nim1,ii+is));
      jb = max(0,min(nim1,ii+is*b));
      ilm1 = il-1; if (ilm1==-1) ilm1 = 0;
      ilp1 = il+1; if (ilp1==nl) ilp1 = nlm1;
      dm = d[ilm1+nl*jb];
      di = d[il+nl*ji];
      dp = d[ilp1+nl*jb];
      for (kb=ji; kb!=jb; kb+=is) {
         dm += e[ilm1+nl*kb];
         dp += e[ilp1+nl*kb];
      }
      dl = min3(dm,di,dp);
      if (dl!=di) {
         if (dl==dm) {
            il = ilm1;
         } else {
            il = ilp1;
         }
      }
      ii += is;
      u[ii] = il+lmin;
      if (il==ilm1 || il==ilp1) {
         for (kb=ji; kb!=jb; kb+=is) {
            ii += is;
            u[ii] = il+lmin;
         }
      }
   }
}

